#include <stdlib.h>

// returns a safe path name for a socket to use in the remote module and
// remote utility.
// if no safe directory can be determined this returns NULL and no remote
// socket must be setup
// otherwise a malloc'd string is returned that needs to be free()'d by the
// caller when it isn't needed any longer.
static inline const char* GetRemoteSocketPath(int display_nr)
{
    const char *hidden = "";
    const char *dir = getenv("XDG_RUNTIME_DIR");
    if (!dir) {
        dir = getenv("HOME");
        // if it is placed in the home directory then add a "." prefix to the
        // basename to make it hidden
        hidden = ".";
    }
    if (!dir) {
        // no safe directory found
        return NULL;
    }

    char *path = NULL;

    if (asprintf(&path, "%s/%sfcitx-socket-:%d", dir, hidden, fcitx_utils_get_display_number()) < 0)
        // formatting error
        return NULL;

    return path;
}
